/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.parser.filter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.List;

import org.tinygroup.parser.Node;
import org.tinygroup.parser.NodeFilter;

public abstract class AbstractFilterImpl<T extends Node<T>> implements
		NodeFilter<T> {
	private Map<String, String> includeAttribute = null;
	private Map<String, String> excludeAttribute = null;
	private String[] includeAttributeV = null;// 包含属性及值
	private String[] excludeAttributeV = null;// 不包含属性及值
	private String[] includeText = null;// 包含文本
	private String[] excludeText = null;// 不包含文本
	private String[] includeNode = null;// 包含子节点
	private String[] excludeNode = null;// 不包含子节点
	private String[] xorSubNode = null;// 至少包含一个指定名称的节点
	private String[] xorProperties = null;// 至少包含一个指定名称属性
	private T node = null;

	public T getNode() {
		return node;
	}

	class NodeList implements Comparable<NodeList> {
		private String nodeName;
		private List<T> nodeList = new ArrayList<T>();

		public List<T> getNodeList() {
			return nodeList;
		}

		NodeList(String nodeName) {
			this.nodeName = nodeName;
		}

		public int compareTo(NodeList o) {
			return nodeName.compareTo(o.nodeName);
		}
	}

	public AbstractFilterImpl(T node) {
		this.node = node;
	}

	public void setXorSubNode(String... xorSubNode) {
		this.xorSubNode = xorSubNode;
	}

	public void setXorProperties(String... xorProperties) {
		this.xorProperties = xorProperties;
	}

	protected boolean isRightNode(T node) {
		boolean right = true;
		if (xorSubNode != null) {
			boolean r = checkXorSubNode(node);
			if (!r) {
				return false;
			}
		}
		if (xorProperties != null) {
			boolean r = checkXorProperties(node);
			if (!r) {
				return false;
			}
		}
		if (includeText != null) {//
			String content = node.getContent();
			if (content == null) {
				return false;
			}
			for (String str : includeText) {
				if (content.indexOf(str) < 0) {
					return false;
				}
			}
		}
		if (excludeText != null) {//
			String content = node.getContent();
			if (content == null) {
				return true;
			}
			for (String str : excludeText) {
				if (content.indexOf(str) >= 0) {
					return false;
				}
			}
		}
		if (includeAttribute != null) {// 所有的属性都要被包含
			for (String key : includeAttribute.keySet()) {
				String value = node.getAttribute(key);
				if (value == null || !value.equals(includeAttribute.get(key))) {
					return false;
				}
			}
		}
		if (excludeAttribute != null) {// 包含了不应有的属性
			for (String key : excludeAttribute.keySet()) {
				String value = node.getAttribute(key);
				if (value != null && value.equals(excludeAttribute.get(key))) {
					return false;
				}
			}
		}
		if (includeAttributeV != null) {// 没有包含应有的属性名
			for (String key : includeAttributeV) {
				if (node.getAttribute(key) == null) {
					return false;
				}
			}
		}
		if (excludeAttributeV != null) {// 包含了不应有的属性名
			for (String key : excludeAttributeV) {
				if (node.getAttribute(key) != null) {
					return false;
				}
			}
		}
		if (includeNode != null) {
			for (String inName : includeNode) {
				List<T> sn = node.getSubNodes();
				if (sn == null) {
					return false;
				} else {
					boolean findInclude = false;
					for (T n : sn) {
						String snn = n.getNodeName();
						if (snn != null && inName.equals(snn)) {
							findInclude = true;
							break;
						}
					}
					if (!findInclude) {
						return false;// 没有包含应有的结点
					}
				}
			}
		}
		if (excludeNode != null) {
			for (String inName : excludeNode) {
				List<T> sn = node.getSubNodes();
				if (sn == null) {
					return true;
				} else {
					boolean findInclude = false;
					for (T n : sn) {
						String snn = n.getNodeName();
						if (snn != null && inName.equals(snn)) {
							findInclude = true;
							break;
						}
					}
					if (findInclude) {
						return false;
					}
				}
			}
		}
		return right;
	}

	private boolean checkXorProperties(T node) {
		boolean r = false;
		for (String nn : xorProperties) {
			if (node.getNodeType().isHasHeader()
					&& node.getAttribute(nn) != null) {
				r = true;
				break;
			}
		}
		return r;
	}

	private boolean checkXorSubNode(T node) {
		boolean r = false;
		for (String nn : xorSubNode) {
			if (node.getSubNodes(nn) != null) {
				r = true;
				break;
			}
		}
		return r;
	}

	/**
	 * 过滤不符合条件的结点
	 * 
	 * @param nodeWithPath
	 * @return List<T>
	 */
	protected List<T> filteNode(List<T> nodeWithPath) {
		List<T> result = new ArrayList<T>();
		for (T tNode : nodeWithPath) {
			if (isRightNode(tNode)) {
				result.add(tNode);
			}
		}
		return result;
	}

	public void setIncludeAttribute(Map<String, String> includeAttribute) {
		if (this.includeAttribute == null) {
			this.includeAttribute = includeAttribute;
		} else {
			this.includeAttribute.putAll(includeAttribute);
		}
	}

	public void setIncludeAttribute(String key, String value) {
		if (includeAttribute == null) {
			includeAttribute = new HashMap<String, String>();
		}
		includeAttribute.put(key, value);
	}

	public void setExcludeAttribute(Map<String, String> excludeAttribute) {
		this.excludeAttribute = excludeAttribute;
	}

	public void setIncludeText(String... includeText) {
		this.includeText = includeText;
	}

	public void setExcludeText(String... excludeText) {
		this.excludeText = excludeText;
	}

	public void setIncludeNode(String... includeNode) {
		this.includeNode = includeNode;
	}

	public void setExcludeNode(String... excludeNode) {
		this.excludeNode = excludeNode;
	}

	public void clearCondition() {
		includeAttribute = null;
		excludeAttribute = null;
		includeText = null;
		excludeText = null;
		includeNode = null;
		excludeNode = null;
		includeAttributeV = null;
		excludeAttributeV = null;
		xorSubNode = null;
		xorProperties = null;
	}

	public void setIncludeAttributes(String... includeAttribute) {
		this.includeAttributeV = includeAttribute;

	}

	public void setExcludeAttribute(String... excludeAttribute) {
		this.excludeAttributeV = excludeAttribute;

	}

	public T findNode(String nodeName) {
		List<T> result = findNodeList(nodeName);
		if (result.size() == 0) {
			return null;
		} else {
			return result.get(0);
		}
	}
}
